/*
 * Copyright 2011 Hanson Robokind LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.robokind.impl.motion.openservo.utils;

import java.util.logging.Level;
import java.util.logging.Logger;
import org.robokind.api.motion.servos.ServoController;
import org.robokind.api.motion.servos.ServoController.ServoId;
import org.robokind.api.motion.servos.utils.ServoIdReader;
import org.robokind.impl.motion.openservo.OpenServo;

/**
 *
 * @author Matthew Stevenson <www.robokind.org>
 */
public class OpenServoIdReader implements 
        ServoIdReader<OpenServo.Id> {
    private final static Logger theLogger = 
            Logger.getLogger(OpenServoIdReader.class.getName());
    
    @Override
    public ServoId<OpenServo.Id> read(
            ServoController.Id controllerId, String servoIdStr) {
        if(controllerId == null || servoIdStr == null){
            throw new NullPointerException();
        }
        OpenServo.Id dId = read(servoIdStr);
        return new ServoId<OpenServo.Id>(controllerId, dId);
    }

    @Override
    public OpenServo.Id read(String servoIdStr) {
        if(servoIdStr == null){
            throw new NullPointerException();
        }
        int splitIndex = servoIdStr.indexOf("::");
        if(splitIndex < 0 || splitIndex >= (servoIdStr.length()-2)){
            throw new IllegalArgumentException();
        }
        String rs485Str = servoIdStr.substring(0, splitIndex).trim();
        String i2cStr = servoIdStr.substring(splitIndex+2).trim();
        try{
            int rs485Id = Integer.parseInt(rs485Str);
            int i2cId = Integer.parseInt(i2cStr);
            return new OpenServo.Id(rs485Id, i2cId);
        }catch(NumberFormatException ex){
            theLogger.log(Level.SEVERE, "Could not read OpenServo.Id", ex);
            throw ex;
        }
    }

    @Override
    public Class<OpenServo.Id> getServoIdClass() {
        return OpenServo.Id.class;
    }
    
}
